# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.dexterity.fti import DexterityFTI
from Products.CMFCore.utils import getToolByName
from zope.component import getSiteManager
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import six


TEST_CONTENT_TYPE_ID = 'TestContentType'

VOCABULARY_TUPLES = [
    (u'first_value', u'First Title'),
    (u'second_value', None),
]

VOCABULARY = SimpleVocabulary(
    [SimpleTerm(value=v, title=t) for (v, t) in VOCABULARY_TUPLES])


def vocabulary_factory(context):
    return VOCABULARY


class DXLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE, )

    def setUpPloneSite(self, portal):
        '''Set up additional products and ZCML required to test
        this product.
        '''
        # setup dexterity
        types_tool = getToolByName(portal, 'portal_types')

        sm = getSiteManager(portal)
        sm.registerUtility(
            component=vocabulary_factory,
            provided=IVocabularyFactory,
            name=u'Products.CMFDiffTool.testing.VOCABULARY'
        )

        fti = DexterityFTI(
            TEST_CONTENT_TYPE_ID,
            factory=TEST_CONTENT_TYPE_ID,
            global_allow=True,
            behaviors=(
                'plone.app.versioningbehavior.behaviors.IVersionable',
                'plone.app.dexterity.behaviors.metadata.IBasic',
                'plone.app.relationfield.behavior.IRelatedItems',
                'plone.app.contenttypes.behaviors.collection.ICollection',
            ),
            model_source='''
            <model xmlns='http://namespaces.plone.org/supermodel/schema'>
                <schema>
                    <field name='text' type='zope.schema.Text'>
                        <title>Text</title>
                        <required>False</required>
                    </field>
                    <field name='file' type='plone.namedfile.field.NamedFile'>
                        <title>File</title>
                        <required>False</required>
                    </field>
                    <field name='date' type='zope.schema.Date'>
                        <title>Date</title>
                        <required>False</required>
                    </field>
                    <field name='files' type='zope.schema.List'>
                        <title>Date</title>
                        <required>False</required>
                        <value_type type='plone.namedfile.field.NamedFile'>
                            <title>Val</title>
                        </value_type>
                    </field>
                    <field name='choice' type='zope.schema.Choice'>
                        <title>Choice</title>
                        <required>False</required>
                        <vocabulary>Products.CMFDiffTool.testing.VOCABULARY</vocabulary>
                    </field>
                </schema>
            </model>
            '''
        )
        types_tool._setObject(TEST_CONTENT_TYPE_ID, fti)

PACKAGE_DX_FIXTURE = DXLayer()

if six.PY2:
    from plone.app.testing.bbb import PTC_FIXTURE

    class ATLayer(PloneSandboxLayer):

        defaultBases = (PTC_FIXTURE, )

        def setUpZope(self, app, configurationContext):
            # setup schema extender if available
            try:
                from archetypes import schemaextender
                self.loadZCML(package=schemaextender)
            except ImportError:
                pass

    PACKAGE_AT_FIXTURE = ATLayer()
    CMFDiffToolATLayer = FunctionalTesting(
        bases=(PACKAGE_AT_FIXTURE, ), name='Products.CMFDiffTool.AT:functional')


CMFDiffToolDXLayer = FunctionalTesting(
    bases=(PACKAGE_DX_FIXTURE, ), name='Products.CMFDiffTool.DX:functional')



# -*- coding: utf-8 -*-
from AccessControl.class_init import InitializeClass
from Products.CMFDiffTool.FieldDiff import FieldDiff
from six.moves import range


def chk_hashable(value):
    try:
        hash(value)
    except TypeError as e:
        value = repr(e) + ": " + repr(value)
    return value


class ListDiff(FieldDiff):
    """Text difference"""

    meta_type = 'List Diff'

    def _parseField(self, value, filename=None):
        """Parse a field value in preparation for diffing"""
        if type(value) is list or type(value) is tuple:
            values = []
            for v in value:
                values.append(chk_hashable(v))
            return values
        else:
            if type(value) is set:
                return list(value)
            else:
                return [chk_hashable(value)]


class RelationListDiff(FieldDiff):

    meta_type = 'Related List Diff'

    same_fmt = """<div class="%s"><a target="_blank" href="%s">%s</a></div>"""
    inlinediff_fmt = """<div class="%s">
        <div class="%s"><a target="_blank" href="%s">%s</a></div>
    </div>"""

    def _parseField(self, value, filename=None):
        """Take RelationValues and just return the target UID so we can compare"""

        if filename is None:
            # Since we only want to compare a single field, make a
            # one-item list out of it
            return ['/'.join(val.getPhysicalPath()) for val in value]
        else:
            return [
                self.filenameTitle(filename),
                ['/'.join(val.getPhysicalPath()) for val in value]
            ]

    def inline_diff(self):
        css_class = 'InlineDiff'
        inlinediff_fmt = self.inlinediff_fmt
        same_fmt = self.same_fmt
        r = []
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == 'replace':
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_title = obj.Title()
                    obj_url = obj.absolute_url()
                    r.append(inlinediff_fmt %
                             (css_class, "diff_sub", obj_url, obj_title))
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_title = obj.Title()
                    obj_url = obj.absolute_url()
                    r.append(inlinediff_fmt %
                             (css_class, "diff_add", obj_url, obj_title))
            elif tag == 'delete':
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_title = obj.Title()
                    obj_url = obj.absolute_url()
                    r.append(inlinediff_fmt %
                             (css_class, "diff_sub", obj_url, obj_title))
            elif tag == 'insert':
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_title = obj.Title()
                    obj_url = obj.absolute_url()
                    r.append(inlinediff_fmt %
                             (css_class, "diff_add", obj_url, obj_title))
            elif tag == 'equal':
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_title = obj.Title()
                    obj_url = obj.absolute_url()
                    r.append(same_fmt % (css_class, obj_url, obj_title))
            else:
                raise ValueError('unknown tag %s' % tag)
        return '\n'.join(r)

    def ndiff(self):
        """Return a textual diff"""
        r = []
        for tag, alo, ahi, blo, bhi in self.getLineDiffs():
            if tag == 'replace':
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_url = obj.absolute_url()
                    r.append("- %s" % obj_url)
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_url = obj.absolute_url()
                    r.append("+ %s" % obj_url)
            elif tag == 'delete':
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_url = obj.absolute_url()
                    r.append("- %s" % obj_url)
            elif tag == 'insert':
                for i in range(blo, bhi):
                    obj = self.newValue[i]
                    obj_url = obj.absolute_url()
                    r.append("+ %s" % obj_url)
            elif tag == 'equal':
                for i in range(alo, ahi):
                    obj = self.oldValue[i]
                    obj_url = obj.absolute_url()
                    r.append("  %s" % obj_url)
            else:
                raise ValueError('unknown tag %r', tag)
        return '\n'.join(r)


InitializeClass(ListDiff)
InitializeClass(RelationListDiff)



# -*- coding: utf-8 -*-
import six

if six.PY2:
    from .BaseTestCase import BaseATTestCase
    from Products.Archetypes import atapi
    from Products.ATContentTypes.content.document import ATDocument
    from Products.CMFDiffTool.ATCompoundDiff import ATCompoundDiff
    from zope.component import adapts
    from zope.component import provideAdapter
    from zope.interface import alsoProvides
    from zope.interface import noLongerProvides


    class TestATCompoundDiff(BaseATTestCase):
        """Test the portal_diff tool"""

        def testCompoundDiff(self):
            first = self.folder.invokeFactory('Document', 'extended-document')
            first = self.folder[first]
            second = self.folder.invokeFactory('Document', 'extended-document2')
            second = self.folder[second]
            # Change a value
            first.setText('<p>Body1</p>', mimetype='text/html')
            second.setText('<p>Body2</p>', mimetype='text/html')
            fd = ATCompoundDiff(first, second, None)
            # There should be some fields
            self.assertTrue(len(fd))
            for field in fd:
                # We've changed the body
                if field.label == 'label_body_text':
                    # We have the correct values
                    self.assertEqual(field.oldValue, '<p>Body1</p>')
                    self.assertEqual(field.newValue, '<p>Body2</p>')
                    # And the correctly assigned diff type
                    self.assertEqual(field.meta_type, 'HTML Diff')
                # We've also changed the id
                elif field.label == 'label_short_name':
                    self.assertEqual(field.oldValue, 'extended-document')
                    self.assertEqual(field.newValue, 'extended-document2')
                    self.assertEqual(field.meta_type, 'Lines Diff')
                # The dates will be different, but everything else should
                # be the same
                elif field.label not in ('label_creation_date',
                                         'label_modification_date'):
                    self.assertEqual(field.oldValue, field.newValue)

        def testAdaptedObjects(self):
            # Add a schema extended field to an ATDocument and test that
            # the compound diff can read it
            from archetypes.schemaextender.field import ExtensionField
            from archetypes.schemaextender.tests.mocks import Extender
            from archetypes.schemaextender.tests.mocks import IHighlighted

            # custom field class which does not set attributes directly on the
            # content
            class HighlightedField(ExtensionField, atapi.StringField):
                def get(self, instance, **kwargs):
                    return IHighlighted.providedBy(instance)

                def getRaw(self, instance, **kwargs):
                    return self.get(instance, **kwargs)

                def set(self, instance, value, **kwargs):
                    if value and not IHighlighted.providedBy(instance):
                        alsoProvides(instance, IHighlighted)
                    elif not value and IHighlighted.providedBy(instance):
                        noLongerProvides(instance, IHighlighted)

            class TestSchemaExtender(Extender):
                adapts(ATDocument)
                fields = [
                    HighlightedField('schemaextender_test',
                                     schemata='settings',
                                     widget=atapi.BooleanWidget(
                                         label='Extended',
                                         description=''),
                                     ),
                ]

            """Ensure that tool instances implement the portal_diff interface"""
            provideAdapter(TestSchemaExtender,
                           name=u"archetypes.schemaextender.tests")

            first = self.folder.invokeFactory('Document', 'extended-document')
            first = self.folder[first]
            second = self.folder.invokeFactory('Document', 'extended-document2')
            second = self.folder[second]
            # Change the value
            alsoProvides(second, IHighlighted)
            fd = ATCompoundDiff(first, second, None)
            field = fd[-1]
            self.assertEqual(field.oldValue, False)
            self.assertEqual(field.newValue, True)
            self.assertEqual(field.label, 'Extended')

else:
    import unittest

    class TestDummy(unittest.TestCase):

        def test_dummy(self):
            return



# -*- coding: utf-8 -*-
from Products.CMFDiffTool.testing import CMFDiffToolDXLayer

import six
import unittest


class BaseDXTestCase(unittest.TestCase):
    """ A base testing class for CMFDiffTool

        It includes a layer which installes the product
        and some testing dependencies in a Plone site.
    """
    layer = CMFDiffToolDXLayer

    def setUp(self):
        self.portal = self.layer['portal']


if six.PY2:
    from plone.app.testing.bbb import PloneTestCase
    from Products.CMFDiffTool.testing import CMFDiffToolATLayer

    class BaseATTestCase(PloneTestCase):
        """ A base testing class for CMFDiffTool

            It includes a layer which installes the product
            and some testing dependencies in a Plone site.
        """
        layer = CMFDiffToolATLayer
